import {
  ActionRowBuilder,
  ButtonBuilder,
  ButtonStyle,
  Client,
  MessageComponentInteraction,
  User,
} from "discord.js";
import { v4 as uuidv4 } from "uuid";
import { getLocale } from "../../utils/locales";
import { getFilename } from "../../utils/misc";
import { embedListReminders } from "../../utils/reminder";
import { collect } from "../loader";

export default {
  data: {
    name: getFilename(__filename),
  },
  interaction: async (interaction: MessageComponentInteraction, client: Client) => {
    const loc = getLocale(client, interaction.locale);
    const embed_desc = interaction.message.embeds.at(0)?.description as string;

    // Retrieve Pages
    const pageMax = Number(/(\d+)(?!.*\d)/gm.exec(embed_desc)?.[0]);
    let page = Number(/(?!• \s+)\d(?=\/)/gm.exec(embed_desc)?.[0]);
    if (page - 1 == 0) {
      page = pageMax;
    } else {
      page--;
    }

    // Retrieve user
    const userId = /(?!<@)\d+(?=>)/gm.exec(embed_desc)?.[0] as string;
    const user = client.users.cache.get(userId) as User;

    // Fetch list
    const list = await embedListReminders(
      client,
      user,
      interaction.guildId,
      page,
      interaction.locale,
    );

    const idPrec = "reminderList-prec_" + uuidv4();
    const idNext = "reminderList-next_" + uuidv4();
    const row = new ActionRowBuilder<ButtonBuilder>()
      .addComponents(
        new ButtonBuilder()
          .setCustomId(idPrec)
          .setLabel(loc.get("c_reminder12"))
          .setStyle(ButtonStyle.Primary),
      )
      .addComponents(
        new ButtonBuilder()
          .setCustomId(idNext)
          .setLabel(loc.get("c_reminder13"))
          .setStyle(ButtonStyle.Primary),
      );

    // Buttons interactions
    collect(client, interaction, idPrec);
    collect(client, interaction, idNext);

    return {
      embeds: [list],
      components: [row],
    };
  },
};
