import { SlashCommandBuilder } from "@discordjs/builders";
import { Locale } from "discord-api-types/v9";
import { ChatInputCommandInteraction, Client, Colors, EmbedBuilder } from "discord.js";
import "../../modules/string";
import { getLocale, getLocalizations } from "../../utils/locales";
import { getFilename } from "../../utils/misc";

export default {
  scope: () => [],

  data: (client: Client) => {
    const filename = getFilename(__filename);
    return (
      new SlashCommandBuilder()
        .setName(filename.toLowerCase())
        .setDescription(
          client.locales.get(client.config.default_lang)?.get(`c_${filename}_desc`) ?? "",
        )
        .setNameLocalizations(getLocalizations(client, `c_${filename}_name`, true))
        .setDescriptionLocalizations(getLocalizations(client, `c_${filename}_desc`))

        // Command option
        .addStringOption((option) =>
          option
            .setName(
              client.locales.get(client.config.default_lang)?.get(`c_${filename}_opt1_name`) ?? "",
            )
            .setDescription(
              client.locales.get(client.config.default_lang)?.get(`c_${filename}_opt1_desc`) ?? "",
            )
            .setNameLocalizations(getLocalizations(client, `c_${filename}_opt1_name`, true))
            .setDescriptionLocalizations(getLocalizations(client, `c_${filename}_opt1_desc`)),
        )
    );
  },

  interaction: async (interaction: ChatInputCommandInteraction, client: Client) => {
    const loc = getLocale(client, interaction.locale);
    const desired_command = interaction.options.getString(
      client.locales
        .get(client.config.default_lang)
        ?.get(`c_${getFilename(__filename)}_opt1_name`) ?? "",
    );

    // If a command isn't specified
    if (!desired_command) {
      const fields: {
        name: string;
        value: string;
      }[] = [];

      // Load all the command per categories
      // TODO: Check if the command exist in the context (guild)
      // TODO: List subcommands too
      client.commands.categories.forEach((commands_name, category) => {
        const commands = commands_name.reduce((data, command_name) => {
          return data + `\`/${command_name}\`, `;
        }, "");

        fields.push({
          name: category.capitalize() + ` (${commands_name.length})`,
          value: commands.slice(0, -2),
        });
      });

      // Sends a list of commands sorted into categories
      return interaction.reply({
        embeds: [
          new EmbedBuilder()
            .setColor(Colors.Blurple)
            .setTitle(loc.get("c_help1"))
            .setDescription(loc.get("c_help2"))
            .addFields(fields),
        ],
      });
    }

    // If a command is specified
    // TODO: Check if the command exist in the context (guild)
    const command = client.commands.list.get(desired_command);
    if (!command) {
      // Command don't exist
      return interaction.reply({
        content: `${loc.get("c_help3")} \`${desired_command}\``,
        ephemeral: true,
      });
    }

    // Send information about the command
    return interaction.reply({
      embeds: [
        new EmbedBuilder()
          .setColor(Colors.Blurple)
          .setTitle("`/" + command.data.name + "`")
          .setDescription(
            // Loads the description
            // according to the user's locals
            command.data.description_localizations?.[interaction.locale as Locale] ??
              command.data.description,
          ),
      ],
    });
  },
};
