import { SlashCommandBuilder } from "@discordjs/builders";
import { Player, useMainPlayer, useQueue } from "discord-player";
import { ChatInputCommandInteraction, Client, EmbedBuilder } from "discord.js";
import { getLocale, getLocalizations } from "../../utils/locales";
import { getFilename } from "../../utils/misc";

export default {
  scope: () => [],

  data: (client: Client) => {
    const filename = getFilename(__filename);
    const loc_default = client.locales.get(client.config.default_lang);
    if (!loc_default) {
      return;
    }

    return (
      new SlashCommandBuilder()
        .setName(filename.toLowerCase())
        .setDescription(loc_default.get(`c_${filename}_desc`) ?? "")
        .setNameLocalizations(getLocalizations(client, `c_${filename}_name`, true))
        .setDescriptionLocalizations(getLocalizations(client, `c_${filename}_desc`))

        // Normal
        .addSubcommand((subcommand) =>
          subcommand
            .setName(loc_default.get(`c_${filename}_sub1_name`)?.toLowerCase() ?? "")
            .setDescription(loc_default.get(`c_${filename}_sub1_desc`) ?? "")
            .setNameLocalizations(getLocalizations(client, `c_${filename}_sub1_name`, true))
            .setDescriptionLocalizations(getLocalizations(client, `c_${filename}_sub1_desc`))

            // Command option
            .addStringOption((option) =>
              option
                .setName(loc_default.get(`c_${filename}_opt1_name`)?.toLowerCase() ?? "")
                .setDescription(loc_default.get(`c_${filename}_opt1_desc`) ?? "")
                .setNameLocalizations(getLocalizations(client, `c_${filename}_opt1_name`, true))
                .setDescriptionLocalizations(getLocalizations(client, `c_${filename}_opt1_desc`)),
            ),
        )

        // Romanized
        .addSubcommand((subcommand) =>
          subcommand
            .setName(loc_default.get(`c_${filename}_sub2_name`)?.toLowerCase() ?? "")
            .setDescription(loc_default.get(`c_${filename}_sub2_desc`) ?? "")
            .setNameLocalizations(getLocalizations(client, `c_${filename}_sub2_name`, true))
            .setDescriptionLocalizations(getLocalizations(client, `c_${filename}_sub2_desc`))

            // Command option
            .addStringOption((option) =>
              option
                .setName(loc_default.get(`c_${filename}_opt1_name`)?.toLowerCase() ?? "")
                .setDescription(loc_default.get(`c_${filename}_opt1_desc`) ?? "")
                .setNameLocalizations(getLocalizations(client, `c_${filename}_opt1_name`, true))
                .setDescriptionLocalizations(getLocalizations(client, `c_${filename}_opt1_desc`)),
            ),
        )
    );
  },

  interaction: async (interaction: ChatInputCommandInteraction, client: Client) => {
    const loc_default = client.locales.get(client.config.default_lang);
    const filename = getFilename(__filename);

    const loc = getLocale(client, interaction.locale);

    let request = interaction.options.getString(
      loc_default?.get(`c_${filename}_opt1_name`) as string,
    );

    let data = null;
    await interaction.deferReply();

    const player = useMainPlayer() as Player;
    if (request) {
      if (
        interaction.options.getSubcommand() ==
          loc_default?.get(`c_${filename}_sub2_name`)?.toLowerCase() ??
        ""
      ) {
        // Romanized
        request += " romanized";
      }

      try {
        data = await player.lyrics.search(request);
      } catch {
        return await interaction.followUp(`❌ | ${loc.get("c_lyrics2")} \`${request}\``);
      }
    } else {
      const queue = useQueue(interaction.guildId ?? "");
      if (queue) {
        const title = queue.history.currentTrack?.title;
        if (title) {
          try {
            data = await player.lyrics.search(title + " " + queue.history.currentTrack?.author);
          } catch {
            return await interaction.followUp(`❌ | ${loc.get("c_lyrics2")} \`${title}\``);
          }
        }
      }
    }

    if (data) {
      const limit_desc = 4096;
      const nb_embed = Math.ceil(data.lyrics.length / limit_desc);

      // TODO: If lyrics < 6000, only send one message with multiples embed
      for (let i = 0, j = 0; i < nb_embed; i++, j += limit_desc) {
        // TODO: Better cut in lyrics
        const lyrics = data.lyrics.slice(j, j + limit_desc);

        let embed;
        switch (i) {
          case 0: {
            // First embed
            embed = new EmbedBuilder();
            embed
              .setTitle(data.title)
              .setURL(data.url)
              .setAuthor({
                name: data?.artist.name,
                iconURL: data?.artist.image,
                url: data?.artist.url,
              })
              .setDescription(lyrics)
              .setThumbnail(data.thumbnail);
            break;
          }

          case nb_embed - 1: {
            // Footer of last embed in case of multiple embed
            embed = new EmbedBuilder().setDescription(lyrics).setFooter({
              text: `${data?.artist.name} · ${data.title}`,
              iconURL: data?.artist.image,
            });
            break;
          }

          default: {
            // Embed with only lyrics in case of multiple embed
            embed = new EmbedBuilder().setDescription(lyrics);
            break;
          }
        }

        // Send embed
        await interaction.followUp({ embeds: [embed] });
      }

      return;
    }

    return await interaction.followUp(`❌ | ${loc.get("c_lyrics1")}`);
  },
};
