import { Player, PlayerEvents, useMainPlayer } from "discord-player";
import { Client } from "discord.js";
import { readdir } from "fs/promises";

/** Load all the events. */
export default async (client: Client) => {
  const events_categories = (await readdir(__dirname)).filter(
    (element) => !element.endsWith(".js") && !element.endsWith(".ts"),
  );

  events_categories.forEach(async (event_category) => {
    // Retrieve events
    const events = await readdir(`${__dirname}/${event_category}`);

    // Load them into the client
    Promise.all(
      events.map(async (event_file) => {
        const { once, default: execute } = await import(
          `../events/${event_category}/${event_file}`
        );

        // Remove extension
        // TODO: use utils functions
        const event_type_ext = event_file.split(".");
        const ext = event_type_ext.pop();
        if (!(ext === "js" || ext === "ts")) {
          throw `Unknown file in ${event_category}: ${event_file}`;
        }
        const event_type = event_type_ext.join(".");

        if (event_category == "player") {
          const player = useMainPlayer() as Player;
          if (once) {
            // eslint-disable-next-line
            return player.events.once(event_type as keyof PlayerEvents, (...args: any[]) => {
              execute(...args, client);
            });
          }
          // eslint-disable-next-line
          return player.events.on(event_type as keyof PlayerEvents, (...args: any[]) => {
            execute(...args, client);
          });
        }

        if (once) {
          return client.once(event_type, (...args) => {
            execute(...args, client);
          });
        }
        return client.on(event_type, (...args) => {
          execute(...args, client);
        });
      }),
    );
  });
};
