import { lyricsExtractor } from "@discord-player/extractor";
import { Player } from "discord-player";
import { Client, Collection, GatewayIntentBits } from "discord.js";
import { readFileSync } from "fs";
import { Database } from "sqlite3";
import "../modules/client";
import { loadLocales } from "./locales";

/** Creation of the client and definition of its properties. */
export default async () => {
  const client: Client = new Client({
    intents: [
      GatewayIntentBits.Guilds,
      GatewayIntentBits.GuildMessages,
      GatewayIntentBits.MessageContent,
      GatewayIntentBits.GuildVoiceStates,
    ],
  });

  client.config = {
    version: JSON.parse(readFileSync("./package.json").toString()).version,
    token_discord: process.env.TOKEN_DISCORD,
    default_lang: process.env.DEFAULT_LANG ?? "fr",
  };

  client.modals = {
    categories: new Collection(),
    list: new Collection(),
  };

  client.buttons = {
    categories: new Collection(),
    list: new Collection(),
  };

  client.commands = {
    categories: new Collection(),
    list: new Collection(),
  };

  const player = Player.singleton(client, {
    ytdlOptions: {
      filter: "audioonly",
      quality: "highestaudio",
    },
  });
  await player.extractors.loadDefault();

  player.lyrics = lyricsExtractor();

  console.log("Translations progression :");
  client.locales = await loadLocales(client.config.default_lang);

  client.db = new Database(`${process.env.DOCKERIZED === "1" ? "/config" : "./config"}/db.sqlite3`);

  initDatabase(client.db);

  return client;
};

/**
 * Quit gracefully the client
 * @param client Client
 */
export const quit = (client: Client) => {
  // Close DB
  client.db.close();

  // Close client
  client.destroy();
};

/**
 * Initalize the database
 * @param db Database
 */
const initDatabase = (db: Database) => {
  // Table for reminders
  db.run(
    "CREATE TABLE IF NOT EXISTS reminder ( \
			id INTEGER PRIMARY KEY, \
			data TEXT, \
			expiration_date TEXT, \
			option_id INTEGER, \
			channel_id TEXT, \
			creation_date TEXT, \
			user_id TEXT, \
			guild_id TEXT, \
			locale TEXT, \
			timeout_id TEXT \
		);",
  );
};
