import { EmbedBuilder } from "@discordjs/builders";
import { GuildQueue, QueueRepeatMode, Track } from "discord-player";
import { Client } from "discord.js";
import { getLocale } from "./locales";

export const embedListQueue = (
  client: Client,
  embed: EmbedBuilder,
  queue: GuildQueue,
  page: number,
  local: string,
) => {
  const loc = getLocale(client, local);
  const tracks = queue.tracks.toArray();

  // Add the current song at the top of the list
  tracks.unshift(queue.history.currentTrack as Track);

  // Limit of discord is 25
  const limit_fields = 25;

  const pageMax = Math.ceil(tracks.length / limit_fields);

  embed.setAuthor({ name: `${loc.get("c_queue1")} • ${loc.get("c_queue7")} ${page}/${pageMax}` });
  embed.setFooter({ text: `${printRepeatMode(queue.repeatMode, loc)}` });

  tracks.slice((page - 1) * limit_fields, page * limit_fields).forEach((t, idx) => {
    const now_playing = idx == 0 && page == 1;
    const name = now_playing
      ? loc.get("c_queue10")
      : (idx == 1 && page == 1) || (idx == 0 && page > 1)
        ? loc.get("c_queue11")
        : "\u200b";
    const idx_track = now_playing ? "" : `${idx + limit_fields * (page - 1)}. `;
    embed.addFields({
      name,
      value: `${idx_track}[${t.title} • ${t.author}](${t.url}) (${t.duration})`,
    });
  });
};

const printRepeatMode = (mode: QueueRepeatMode, loc: Map<string, string>) => {
  switch (mode) {
    case QueueRepeatMode.OFF:
      return loc.get("c_repeat2");

    case QueueRepeatMode.QUEUE:
      return loc.get("c_repeat3") + " " + loc.get("c_repeat6");

    case QueueRepeatMode.AUTOPLAY:
      return loc.get("c_repeat4") + " " + loc.get("c_repeat6");

    case QueueRepeatMode.TRACK:
      return loc.get("c_repeat5") + " " + loc.get("c_repeat6");

    default:
      break;
  }
};
